@@ -24,6 +24,12 @@ from mvpa2.support.copy import copy
 from mvpa2.datasets.base import Dataset
 from mvpa2.base.collections import ArrayCollectable
 from mvpa2.datasets.base import dataset_wizard
+from mvpa2.mappers.flatten import ProductFlattenMapper
+
+import itertools
+import operator
+
+from mvpa2.base import externals
 
 # arbitrary ndarray subclass for testing
 class myarray(np.ndarray):
@@ -34,7 +40,7 @@ def test_flatten():
     data_shape = (4,) + samples_shape
     data = np.arange(np.prod(data_shape)).reshape(data_shape).view(myarray)
     pristinedata = data.copy()
-    target = [[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15],
+    target = [[ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15],
               [16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31],
               [32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47],
               [48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63]]
@@ -115,10 +121,100 @@ def test_flatten():
         assert_false('voxel' in revds.fa)
 
 
+def test_product_flatten():
+    nsamples = 17
+    product_name_values = [('chan', ['C1', 'C2']),
+                         ('freq', np.arange(4, 20, 6)),
+                         ('time', np.arange(-200, 800, 200))]
+
+    shape = (nsamples,) + tuple(len(v) for _, v in product_name_values)
+
+    sample_names = ['samp%d' % i for i in xrange(nsamples)]
+
+    # generate random data in four dimensions
+    data = np.random.normal(size=shape)
+    ds = Dataset(data, sa=dict(sample_names=sample_names))
+    for n, v in product_name_values:
+        ds.a[n] = v
+
+    # apply flattening to ds
+    names, values = zip(*(product_name_values))
+
+    flattened_ds = None
+
+    # test both with explicit values for factor_values and without
+    for with_values in (False, True):
+        # the order of False and True is critical.
+        # In the first iteration flattened_ds is set and used in the second
+        # iteration
+        args = {}
+        if with_values:
+            factor_values = [v for n, v in product_name_values]
+            args['factor_values'] = factor_values
+
+        flattener = ProductFlattenMapper(names, **args)
+
+    # test I/O (only if h5py is available)
+    if externals.exists('h5py'):
+        from mvpa2.base.hdf5 import h5save, h5load
+        import tempfile
+        import os
+
+        fd, testfn = tempfile.mkstemp('mapper.h5py', 'test_product'); os.close(fd)
+        h5save(testfn, flattener)
+        flattener = h5load(testfn)
+        os.unlink(testfn)
+
+    if flattened_ds is None:
+        assert_raises(ValueError, flattener.reverse, ds)
+    else:
+        ds_ = flattener.reverse(flattened_ds)
+        assert_equal(ds.samples, ds_.samples)
+
+    mds = flattener(ds)
+
+    prod = lambda x:reduce(operator.mul, x)
+
+    # ensure the size is ok
+    assert_equal(mds.shape, (nsamples,) + (prod(shape[1:]),))
+
+    ndim = len(product_name_values)
+
+    idxs = [range(len(v)) for v in values]
+    for si in xrange(nsamples):
+        for fi, p in enumerate(itertools.product(*idxs)):
+            data_tup = (si,) + p
+
+            x = mds[si, fi]
+
+            # value should match
+            assert_equal(data[data_tup], x.samples[0, 0])
+
+            # indices should match as well
+            all_idxs = tuple(x.fa['chan_freq_time_indices'].value.ravel())
+            assert_equal(p, all_idxs)
+
+            # values and indices in each dimension should match
+            for i, (name, value) in enumerate(product_name_values):
+                assert_equal(x.fa[name].value, value[p[i]])
+                assert_equal(x.fa[name + '_indices'].value, p[i])
+
+    dsr = flattener.reverse(mds)
+    assert_equal(dsr.shape, ds.shape)
+
+
+    names += ('foo',)
+
+    flattener = ProductFlattenMapper(names)
+    assert_raises(KeyError, flattener, ds)
+
+    # for next iterations
+    flattened_ds = mds
+
 
 def test_subset():
     data = np.array(
-            [[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15],
+            [[ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15],
             [16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31],
             [32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47],
             [48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63]])
@@ -145,10 +241,10 @@ def test_subset():
     sms = [sm_none, sm_int, sm_bool]
 
     # test subsets
-    sids = [3,4,5,6]
+    sids = [3, 4, 5, 6]
     bsubset = np.zeros(16, dtype='bool')
     bsubset[sids] = True
-    subsets = [sids, slice(3,7), bsubset, [3,3,4,4,6,6,6,5]]
+    subsets = [sids, slice(3, 7), bsubset, [3, 3, 4, 4, 6, 6, 6, 5]]
     # all test subset result in equivalent masks, hence should do the same to
     # the mapper and result in identical behavior
     for st in sms:
@@ -184,7 +280,7 @@ def test_subset():
     # intended merge failures
     fsm = StaticFeatureSelection(np.arange(16))
     assert_equal(fsm.__iadd__(None), NotImplemented)
-    assert_equal(fsm.__iadd__(Dataset([2,3,4])), NotImplemented)
+    assert_equal(fsm.__iadd__(Dataset([2, 3, 4])), NotImplemented)
 
 
 def test_subset_filler():
@@ -232,7 +328,7 @@ def test_chainmapper():
     data_shape = (4,) + samples_shape
     data = np.arange(np.prod(data_shape)).reshape(data_shape)
     pristinedata = data.copy()
-    target = [[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15],
+    target = [[ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15],
               [16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31],
               [32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47],
               [48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63]]
@@ -242,11 +338,11 @@ def test_chainmapper():
     cm.train(data)
 
     # a new mapper should appear when doing feature selection
-    cm.append(StaticFeatureSelection(range(1,16)))
+    cm.append(StaticFeatureSelection(range(1, 16)))
     assert_equal(cm.forward1(data[0]).shape, (15,))
     assert_equal(len(cm), 2)
     # multiple slicing
-    cm.append(StaticFeatureSelection([9,14]))
+    cm.append(StaticFeatureSelection([9, 14]))
     assert_equal(cm.forward1(data[0]).shape, (2,))
     assert_equal(len(cm), 3)
 
@@ -269,7 +365,7 @@ def test_chainmapper():
 
     # let's map something
     mdata = cm.forward(data)
-    assert_array_equal(mdata, target[:,[10,15]])
+    assert_array_equal(mdata, target[:, [10, 15]])
     # and back
     rdata = cm.reverse(mdata)
     # original shape
@@ -296,7 +392,7 @@ def test_sampleslicemapper():
 
 def test_strip_boundary():
     ds = datasets['hollow']
-    ds.sa['btest'] = np.repeat([0,1], 20)
+    ds.sa['btest'] = np.repeat([0, 1], 20)
     sn = StripBoundariesSamples('btest', 1, 2)
     sds = sn(ds)
     assert_equal(len(sds), len(ds) - 3)
@@ -305,7 +401,7 @@ def test_strip_boundary():
 
 def test_transpose():
     from mvpa2.mappers.shape import TransposeMapper
-    ds = Dataset(np.arange(24).reshape(2,3,4),
+    ds = Dataset(np.arange(24).reshape(2, 3, 4),
                  sa={'testsa': np.arange(2)},
                  fa={'testfa': np.arange(3)})
     tp = TransposeMapper()
@@ -330,7 +426,7 @@ def test_transpose():
 
 def test_addaxis():
     from mvpa2.mappers.shape import AddAxisMapper
-    ds = Dataset(np.arange(24).reshape(2,3,4),
+    ds = Dataset(np.arange(24).reshape(2, 3, 4),
                  sa={'testsa': np.arange(2)},
                  fa={'testfa': np.arange(3)})
     ds0 = AddAxisMapper(pos=0)(ds)
